@@ -23,32 +23,55 @@ set -o pipefail
 
 cd ${ROOT_DIR}
 
+# We need these flags for things to work properly.
+export GO111MODULE=on
+
+# This controls the release branch we track.
+VERSION="release-0.17"
+
 # The list of dependencies that we track at HEAD and periodically
 # float forward in this repository.
 FLOATING_DEPS=(
-  "knative.dev/pkg"
-  "knative.dev/caching"
-  "knative.dev/test-infra"
+  "knative.dev/test-infra@${VERSION}"
+  "knative.dev/pkg@${VERSION}"
+  "knative.dev/networking@${VERSION}"
+  "knative.dev/caching@${VERSION}"
 )
 
 # Parse flags to determine any we should pass to dep.
-DEP_FLAGS=()
+GO_GET=0
 while [[ $# -ne 0 ]]; do
   parameter=$1
   case ${parameter} in
-    --upgrade) DEP_FLAGS=( -update ${FLOATING_DEPS[@]} ) ;;
+    --upgrade) GO_GET=1 ;;
     *) abort "unknown option ${parameter}" ;;
   esac
   shift
 done
-readonly DEP_FLAGS
+readonly GO_GET
 
-# Ensure we have everything we need under vendor/
-dep ensure ${DEP_FLAGS[@]}
+if (( GO_GET )); then
+  go get -d ${FLOATING_DEPS[@]}
+fi
 
-rm -rf $(find vendor/ -name 'OWNERS')
-rm -rf $(find vendor/ -name '*_test.go')
 
-update_licenses third_party/VENDOR-LICENSE "./cmd/*"
+# Prune modules.
+go mod tidy
+go mod vendor
 
+echo "Removing unwanted vendor files"
+
+# Remove unwanted vendor files
+find vendor/ \( -name "OWNERS" -o -name "*_test.go" \) -print0 | xargs -0 rm -f
+
+export GOFLAGS=-mod=vendor
+
+echo "Updating licenses"
+update_licenses third_party/VENDOR-LICENSE "./..."
+
+echo "Removing broken symlinks"
 remove_broken_symlinks ./vendor
+
+# Patch k8s leader-election fixing graceful release
+# More information: https://github.com/kubernetes/kubernetes/pull/91942
+git apply ${ROOT_DIR}/hack/k8s-client-go.patch
